#ifndef INDII_CLUSTER_DATASET_HPP
#define INDII_CLUSTER_DATASET_HPP

#include "boost/numeric/ublas/vector.hpp"

#include <map>

namespace indii {
  namespace cluster {

/**
 * Weighted data set for clustering.
 *
 * @author Lawrence Murray <lawrence@indii.org>
 * @version $Rev: 12 $
 * @date $Date: 2007-08-10 01:27:29 +0100 (Fri, 10 Aug 2007) $
 *
 * @param T Datum type.
 * @param W Weight type.
 */
template <class T = double, class W = double>
class DataSet {
public:
  /**
   * Vector comparison for map.
   */
  struct vectorCmp {
    bool operator()(boost::numeric::ublas::vector<T> x1,
        boost::numeric::ublas::vector<T> x2) const {
      typename boost::numeric::ublas::vector<T>::iterator iter1, iter2, end1;

      iter1 = x1.begin();
      iter2 = x2.begin();
      end1 = x1.end();
      while (iter1 != end1) {
        if (*iter1 == *iter2) {
          iter1++;
          iter2++;
        } else {
          return *iter1 < *iter2;
        }
      }
      return false;
    }
  };

  /**
   * Data set.
   */
  typedef std::map<boost::numeric::ublas::vector<T>,W,vectorCmp> data_set;

  /**
   * Data set iterator.
   */
  typedef typename data_set::iterator data_set_iterator;

  /**
   * Data set constant iterator.
   */
  typedef typename data_set::const_iterator data_set_const_iterator;

  /**
   * Constructor.
   *
   * @param N Number of dimensions.
   */
  DataSet(unsigned int N);

  /**
   * Destructor.
   */
  virtual ~DataSet();

  /**
   * Get dimensionality of data set.
   *
   * @return Dimensionality of data set.
   */
  unsigned int getDimensions() const;

  /**
   * Get number of points in data set.
   *
   * @return Number of points in data set.
   */
  unsigned int getSize() const;

  /**
   * Add datum.
   *
   * @param x The datum.
   * @param w Unnormalized weight of the datum.
   */
  void add(const boost::numeric::ublas::vector<T>& x, const W w = 1);

  /**
   * Get iterator to the beginning of the data set.
   *
   * @return Iterator to the beginning of the data set.
   */
  data_set_const_iterator begin() const;

  /**
   * Get iterator to the end of the data set.
   *
   * @return Iterator to the end of the data set.
   */
  data_set_const_iterator end() const;

private:
  /**
   * Number of dimensions.
   */
  const unsigned int N;

  /**
   * Data set.
   */
  data_set data;

};

  }
}

template <class T, class W>
indii::cluster::DataSet<T,W>::DataSet(unsigned int N) : N(N) {
  //
}

template <class T, class W>
indii::cluster::DataSet<T,W>::~DataSet() {
  //
}

template <class T, class W>
unsigned int indii::cluster::DataSet<T,W>::getDimensions() const {
  return N;
}

template <class T, class W>
unsigned int indii::cluster::DataSet<T,W>::getSize() const {
  return data.size();
}

template <class T, class W>
void indii::cluster::DataSet<T,W>::add(
    const boost::numeric::ublas::vector<T>& x, const W w) {
  /* pre-condition */
  assert (x.size() == N);

  data_set_iterator find = data.find(x);
  if (find != data.end()) {
    find->second += w;
  } else {
    data.insert(make_pair(x, w));
  }
}

template <class T, class W>
typename indii::cluster::DataSet<T,W>::data_set_const_iterator indii::cluster::DataSet<T,W>::begin() const {
  return data.begin();
}

template <class T, class W>
typename indii::cluster::DataSet<T,W>::data_set_const_iterator indii::cluster::DataSet<T,W>::end() const {
  return data.end();
}

#endif
