#ifndef INDII_CLUSTER_DISTANCE_HPP
#define INDII_CLUSTER_DISTANCE_HPP

#include "boost/numeric/ublas/vector.hpp"

namespace indii {
  namespace cluster {

/**
 * %Distance functions.
 *
 * @author Lawrence Murray <lawrence@indii.org>
 * @version $Rev: 12 $
 * @date $Date: 2007-08-10 01:27:29 +0100 (Fri, 10 Aug 2007) $
 */
class Distance {
public:
  /**
   * %Distance by Pearson correlation.
   *
   * @param x1 First point.
   * @param x2 Second point, should have same size as first.
   *
   * @return %Distance between the two points.
   */
  template <class T>
  static double distanceByPearson(const boost::numeric::ublas::vector<T>& x1,
      const boost::numeric::ublas::vector<T>& x2);

};

  }
}

template <class T>
double indii::cluster::Distance::distanceByPearson(
    const boost::numeric::ublas::vector<T>& x1,
    const boost::numeric::ublas::vector<T>& x2) {
  /* pre-condition */
  assert (x1.size() == x2.size());

  unsigned int n = x1.size();
  double r, sum1 = 0.0, sum2 = 0.0, sd1, sd2;

  r = boost::numeric::ublas::inner_prod(x1,x2);
  sd1 = boost::numeric::ublas::inner_prod(x1,x1);
  sd2 = boost::numeric::ublas::inner_prod(x2,x2);
  sum1 = boost::numeric::ublas::sum(x1);
  sum2 = boost::numeric::ublas::sum(x2);

  r -= sum1*sum2 / n;
  sd1 -= sum1*sum1 / n;
  sd2 -= sum2*sum2 / n;

  if (sd1 <= 0.0 || sd2 <= 0.0) {
    r = 0.0;
  } else {
    r /= sqrt(sd1*sd2);
  }
  r = 1.0 - r;

  return r;
}

#endif
