#ifndef INDII_CLUSTER_KMEANSCLUSTERER_HPP
#define INDII_CLUSTER_KMEANSCLUSTERER_HPP

#include "DataSet.hpp"

#include "boost/numeric/ublas/vector.hpp"
#include "boost/random.hpp"

#include <vector>

namespace indii {
  namespace cluster {
/**
 * K-means clustering algorithm.
 *
 * @author Lawrence Murray <lawrence@indii.org>
 * @version $Rev: 12 $
 * @date $Date: 2007-08-10 01:27:29 +0100 (Fri, 10 Aug 2007) $
 *
 * @param T Datum element type.
 * @param W Weight type.
 * @param K Cluster index type.
 */
template <class T = double, class W = double, class K = unsigned int>
class KMeansClusterer {
public:
  /**
   * Constructor
   *
   * @param k Number of clusters.
   */
  KMeansClusterer(K k);

  /**
   * Destructor.
   */
  virtual ~KMeansClusterer();

  /**
   * Perform steps of the clustering algorithm until convergence or a
   * maximum number of iterations is reached.
   *
   * @param data Data set to cluster.
   * @param maxIters Maximum number of iterations to perform, zero for
   * no limit (use with caution!).
   */
  virtual void cluster(const DataSet<T,W>& data, unsigned int maxIters = 0);

  /**
   * Determine the nearest cluster for a given point.
   *
   * @param x The point. This may be arbitrary, it needn't have been used in
   * forming the current set of clusters.
   * @param If given, on return, gives the distance of the datum from its
   * assigned cluster centroid.
   *
   * @return Nearest cluster for the given point.
   */
  virtual K assign(const boost::numeric::ublas::vector<T>& x,
      double* distance = NULL) const;

  /**
   * Get estimate of error inherent in the current clustering.
   *
   * @return Error of the current clustering.
   */
  double getError();

private:
  /**
   * Type of centroids.
   */
  typedef boost::numeric::ublas::vector<double> centroid;

  /**
   * Calculate centroids.
   *
   * @param data Data set.
   * @param clusters Assignment of points in data set to clusters.
   */
  void calculateCentroids(const DataSet<T,W>& data,
      const std::vector<K>& clusters);

  /**
   * Number of clusters.
   */
  const unsigned short k;

  /**
   * Error inherent in current clustering.
   */
  double error;

  /**
   * Centroids.
   */
  std::vector<boost::numeric::ublas::vector<double> > centroids;

  /*
   * Random number generation.
   */
  typedef boost::uniform_int<> RandomClusterDist;
  typedef boost::mt19937 RandomClusterRNG;
  typedef boost::variate_generator<RandomClusterRNG&, RandomClusterDist>
      RandomClusterGen;

  RandomClusterDist dist;
  RandomClusterRNG rng;
  RandomClusterGen randomCluster;

};

  }
}

#include "Distance.hpp"

template <class T, class W, class K>
indii::cluster::KMeansClusterer<T,W,K>::KMeansClusterer(K k) :
    k(k), dist(0, k-1), rng(), randomCluster(rng,dist) {
  rng.seed(time(0));
}

template <class T, class W, class K>
indii::cluster::KMeansClusterer<T,W,K>::~KMeansClusterer() {
  //
}

template <class T, class W, class K>
void indii::cluster::KMeansClusterer<T,W,K>::cluster(const DataSet<T,W>& data,
    unsigned int maxIters) {
  unsigned int i, size = data.getSize(), iters = 0, changes = 1;
  double error = 0.0, distance;
  K newK, currentK;
  std::vector<K> clusters;
  typename std::vector<K>::iterator clustersIter;
  typename DataSet<T,W>::data_set_const_iterator dataIter, dataEnd;

  /* allocate space for centroids */
  boost::numeric::ublas::vector<double> x(data.getDimensions());
  centroids.clear();
  for (i = 0; i < k; i++) {
    centroids.push_back(x);
  }

  /* initialise with random clustering */
  for (i = 0; i < size; i++) {
    clusters.push_back(static_cast<K>(randomCluster()));
  }
  calculateCentroids(data, clusters);

  /* iterate */
  while (changes > 0 && iters < maxIters) {
    changes = 0;
    error = 0.0;

    /* assign data to nearest centroid */
    clustersIter = clusters.begin();
    dataIter = data.begin();
    dataEnd = data.end();
    while (dataIter != dataEnd) {
      currentK = *clustersIter;
      newK = assign(dataIter->first, &distance);
      error += distance*distance;
      if (newK != currentK) {
        changes++;
        *clustersIter = newK;
      }

      clustersIter++;
      dataIter++;
    }

    /* recalculate centroids */
    calculateCentroids(data, clusters);

    iters++;
  }
}

template <class T, class W, class K>
K indii::cluster::KMeansClusterer<T,W,K>::assign(
    const boost::numeric::ublas::vector<T>& x, double* distance) const {
  /* pre-condition */
  assert (centroids.size() == k);

  unsigned int i = 0;
  double d = indii::cluster::Distance::distanceByPearson(centroids[i], x);
  unsigned int imin = i;
  double dmin = d;
  for (i = 1; i < k; i++) {
    d = indii::cluster::Distance::distanceByPearson(centroids[i], x);
    if (d < dmin) {
      dmin = d;
      imin = i;
    }
  }

  if (distance != NULL) {
    *distance = dmin;
  }
  return imin;
}

template <class T, class W, class K>
double indii::cluster::KMeansClusterer<T,W,K>::getError() {
  return error;
}

template <class T, class W, class K>
void indii::cluster::KMeansClusterer<T,W,K>::calculateCentroids(
    const DataSet<T,W>& data, const std::vector<K>& clusters) {
  /* pre-condition */
  assert (data.getSize() == clusters.size());

  std::vector<double> centroidWeights;
  typename std::vector<centroid>::iterator centroidsIter, centroidsEnd;
  typename std::vector<double>::iterator centroidWeightsIter;
  typename DataSet<T,W>::data_set_const_iterator dataIter, dataEnd;
  typename std::vector<K>::const_iterator clustersIter;
  unsigned int i;

  /* initialise centroids to zero */
  centroidsIter = centroids.begin();
  centroidsEnd = centroids.end();
  while (centroidsIter != centroidsEnd) {
    centroidsIter->clear();
    centroidsIter++;
  }

  /* initialise centroid weights to zero */
  for (i = 0; i < k; i++) {
    centroidWeights.push_back(0);
  }

  /* calculate centroids as means */
  clustersIter = clusters.begin();
  dataIter = data.begin();
  dataEnd = data.end();
  while (dataIter != dataEnd) {
    noalias(centroids[*clustersIter]) += dataIter->first * dataIter->second;
    centroidWeights[*clustersIter] += dataIter->second;

    dataIter++;
    clustersIter++;
  }

  centroidsIter = centroids.begin();
  centroidsEnd = centroids.end();
  centroidWeightsIter = centroidWeights.begin();
  while (centroidsIter != centroidsEnd) {
    if (*centroidWeightsIter > 0.0) {
      *centroidsIter /= *centroidWeightsIter;
    }
    centroidsIter++;
    centroidWeightsIter++;
  }
}

#endif
