#include "ClusterImage.hpp"

#include "boost/numeric/ublas/vector.hpp"

#include <stdlib.h>
#include <assert.h>
#include <limits>

#define MAX_PIXELS 1e6
#define MAX_ITERS 30

namespace ublas = boost::numeric::ublas;

using namespace indii::tint;
using namespace indii::cluster;

ClusterImage::ClusterImage(wxImage* image) :
    wxImage(image->ConvertToGreyscale()) {
  this->original = image;
  this->k = 1;
  this->doColour = true;
  this->doX = false;
  this->doY = false;
  this->error = std::numeric_limits<double>::infinity();
  this->hasPath = false;
  this->hasBeenSaved = false;
  this->data = NULL;
  this->clusterer = NULL;

  /* jpeg quality setting */
  this->SetOption(_T("quality"), 95);
}

ClusterImage::~ClusterImage() {
  delete data;
  delete clusterer;
}

bool ClusterImage::IsColourConsidered() {
  return this->doColour;
}

bool ClusterImage::IsXConsidered() {
  return this->doX;
}

bool ClusterImage::IsYConsidered() {
  return this->doY;
}

void ClusterImage::SetColourConsidered(bool value) {
  this->doColour = value;
}

void ClusterImage::SetXConsidered(bool value) {
  this->doX = value;
}

void ClusterImage::SetYConsidered(bool value) {
  this->doY = value;
}

void ClusterImage::Prepare() {
  /* clear up any previous preparations */
  delete data;
  delete clusterer;
  data = NULL;
  clusterer = NULL;

  /* prepare pixel vector indices */
  dimensions = 0;
  if (doColour) {
    indexR = dimensions;
    dimensions++;
    indexG = dimensions;
    dimensions++;
    indexB = dimensions;
    dimensions++;
  }
  if (doX) {
    indexX = dimensions;
    dimensions++;
  }
  if (doY) {
    indexY = dimensions;
    dimensions++;
  }
  
  /* prepare working image for clustering */
  int width = original->GetWidth(), height = original->GetHeight();
  int workingWidth, workingHeight;
  wxImage* working;
  double factor = MAX_PIXELS / static_cast<double>(width*height);
  
  if (factor > 1.0) {
    /* image size doesn't exceed limit, cluster it directly */
    workingWidth = width;
    workingHeight = height;
    working = original;
  } else {
    /* image size exceeds limit, cluster scaled version */
    workingWidth = static_cast<int>(width*sqrt(factor));
    workingHeight = static_cast<int>(height*sqrt(factor));
    working = new wxImage(original->Scale(workingWidth, workingHeight));
  }
  
  /* prepare data set for clustering */
  data = new DataSet<double,unsigned int>(dimensions);
  ublas::vector<double> pixel(dimensions);
  int i, x, y;
  
  for (x = 0; x < workingWidth; x++) {
    for (y = 0; y < workingHeight; y++) {
      i = x*workingHeight+y;
      if (doColour) {
        pixel(indexR) = working->GetRed(x,y) / 255.0;
        pixel(indexG) = working->GetGreen(x,y) / 255.0;
        pixel(indexB) = working->GetBlue(x,y) / 255.0;
      }
      if (doX) {
        pixel(indexX) = x / static_cast<double>(workingWidth);
      }
      if (doY) {
        pixel(indexY) = y / static_cast<double>(workingHeight);
      }
      data->add(pixel);
    }
  }
  
  /* clean up */
  if (working != original) {
    delete working;
  }
}

void ClusterImage::Cluster(unsigned int k) {
  KMeansClusterer<double,unsigned int>* clusterer;
  double error;

  /* cluster */
  clusterer = new KMeansClusterer<double,unsigned int>(k);
  clusterer->cluster(*data, MAX_ITERS);
  error = clusterer->getError();
  if (error < this->error) {
    this->k = k;
    this->error = error;
    delete this->clusterer;
    this->clusterer = clusterer;
  } else {
    delete clusterer;
  }
}

void ClusterImage::Finalise() {
  unsigned int x, y, i;
  unsigned int width = original->GetWidth();
  unsigned int height = original->GetHeight();
  ublas::vector<double> pixel(dimensions);
 
  /* allocate space for cluster assignments */
  clusterXs.clear();
  clusterYs.clear();
  for (i = 0; i < k; i++) {
    clusterXs.push_back(std::vector<unsigned int>());
    clusterYs.push_back(std::vector<unsigned int>());
  }

  /* assign pixels of original image to clusters */
  for (x = 0; x < width; x++) {
    for (y = 0; y < height; y++) {
      if (doColour) {
        pixel(indexR) = original->GetRed(x,y) / 255.0;
        pixel(indexG) = original->GetGreen(x,y) / 255.0;
        pixel(indexB) = original->GetBlue(x,y) / 255.0;
      }
      if (doX) {
        pixel(indexX) = x / static_cast<double>(width);
      }
      if (doY) {
        pixel(indexY) = y / static_cast<double>(height);
      }
      i = clusterer->assign(pixel);
      clusterXs[i].push_back(x);
      clusterYs[i].push_back(y);
    }
  }

  /* clean up */
  delete data;
  delete clusterer;
  data = NULL;
  clusterer = NULL;
}

int ClusterImage::GetNumClusters() {
  return k;
}

void ClusterImage::ShowCluster(unsigned int i, bool show) {
  /* pre-condition */
  assert (i < k);

  int x, y, r, g, b, l;
  std::vector<unsigned int>::const_iterator xsIter, xsEnd, ysIter;

  xsIter = clusterXs[i].begin();
  xsEnd = clusterXs[i].end();
  ysIter = clusterYs[i].begin();
  while (xsIter != xsEnd) {
    x = *xsIter;
    y = *ysIter;
    r = original->GetRed(x,y);
    g = original->GetGreen(x,y);
    b = original->GetBlue(x,y);
    if (show) {
      SetRGB(x,y,r,g,b);
    } else {
      l = static_cast<int>(0.299*r + 0.587*g + 0.114*b);
      SetRGB(x,y,l,l,l);
    }
    xsIter++;
    ysIter++;
  }

  /* update state */
  hasBeenSaved = false;
}

bool ClusterImage::SaveFile(const wxString& path) {
  bool success = wxImage::SaveFile(path);
  if (success) {
    this->path = path;
    this->hasPath = true;
    this->hasBeenSaved = true;
  }
  return success;
}

bool ClusterImage::SaveFile() {
  /* pre-condition */
  assert(HasPath());

  return SaveFile(GetPath());
}

bool ClusterImage::HasPath() {
  return hasPath;
}

bool ClusterImage::HasBeenSaved() {
  return hasBeenSaved;
}

const wxString& ClusterImage::GetPath() {
  return path;
}
