#ifndef INDII_TINT_CLUSTERIMAGE_HPP
#define INDII_TINT_CLUSTERIMAGE_HPP

#include "../cluster/DataSet.hpp"
#include "../cluster/KMeansClusterer.hpp"

#include "wx/image.h"

#include <vector>

namespace indii {
  namespace tint {
/**
 * Pixel-clustered image.
 *
 * @author Lawrence Murray <lawrence@indii.org>
 * @version $Rev: 12 $
 * @date $Date: 2007-08-10 01:27:29 +0100 (Fri, 10 Aug 2007) $
 */
class ClusterImage : public wxImage {
public:
  /**
   * Constructor.
   *
   * @param image Original image.
   */
  ClusterImage(wxImage* image);

  /**
   * Destructor.
   */
  virtual ~ClusterImage();

  /**
   * Is colour considered when clustering?
   */
  bool IsColourConsidered();

  /**
   * Is the x-coordinate considered when clustering?
   */
  bool IsXConsidered();
    
  /**
   * Is the y-coordinate considered when clustering?
   */
  bool IsYConsidered();

  /**
   * Set whether colour is considered when clustering.
   *
   * @param value True if colour should be considered, false otherwise.
   */
  void SetColourConsidered(bool value = true);

  /**
   * Set whether the x-coordinate is considered when clustering.
   *
   * @param value True if the x-coordinate should be considered, false
   * otherwise.
   */
  void SetXConsidered(bool value = true);

  /**
   * Set whether the y-coordinate is considered when clustering.
   *
   * @param value True if the y-coordinate should be considered, false
   * otherwise.
   */
  void SetYConsidered(bool value = true);

  /**
   * Prepare image for clustering. Should be called before any calls to
   * Cluster().
   */
  void Prepare();

  /**
   * Perform pixel clustering.
   *
   * @param k Number of clusters.
   *
   * This method may be called multiple times. Each time it is called, pixels
   * are clustered again, but the new clusters are kept only if they are
   * considered an improvement on the existing set of clusters.
   */
  void Cluster(unsigned int k);

  /**
   * Finalise image after clustering. Should be called after one or more calls
   * to Cluster().
   */
  void Finalise();

  /**
   * Get number of clusters.
   *
   * @return Number of clusters.
   */
  int GetNumClusters();

  /**
   * Set whether a cluster should be shown in colour.
   *
   * @param i The cluster number.
   * @param show True if the cluster should be shown in colour, false
   * otherwise.
   */
  void ShowCluster(unsigned int i, bool show = true);

  /**
   * Save the image to a file. If the save is successful, the image's
   * current path is updated to that given.
   *
   * @param path Path to file.
   *
   * @return True if the save is successful, false otherwise.
   */
  bool SaveFile(const wxString& path);

  /**
   * Save the image to the current path.
   *
   * @return True if the save is successful, false otherwise.
   */
  bool SaveFile();

  /**
   * Does the image have a current path?
   *
   * @return True if the image has a current path, false otherwise.
   */
  bool HasPath();

  /**
   * Get the current path of the image.
   *
   * @return The current path of the image.
   */
  const wxString& GetPath();

  /**
   * Has the image been saved since its last change?
   *
   * @return True if the image has been saved since its last change, false
   * otherwise.
   */
  bool HasBeenSaved();

private:
  /**
   * Original image.
   */
  wxImage* original;

  /**
   * Data set for clustering.
   */
  indii::cluster::DataSet<double,unsigned int>* data;

  /**
   * Dimensionality of data points.
   */
  unsigned int dimensions;

  /**
   * Clusterer.
   */
  indii::cluster::KMeansClusterer<double,unsigned int>* clusterer;

  /**
   * Number of clusters.
   */
  unsigned int k;

  /**
   * Pixel cluster assignments, x-coordinates.
   */
  std::vector<std::vector<unsigned int> > clusterXs;

  /**
   * Pixel cluster assignments, y-coordinates.
   */
  std::vector<std::vector<unsigned int> > clusterYs;

  /**
   * Is colour considered when clustering?
   */
  bool doColour;

  /**
   * Is the x-coordinate considered when clustering?
   */
  bool doX;
    
  /**
   * Is the y-coordinate considered when clustering?
   */
  bool doY;

  /**
   * Index of the red component in pixel vectors.
   */
  int indexR;

  /**
   * Index of the green component in pixel vectors.
   */
  int indexG;

  /**
   * Index of the blue component in pixel vectors.
   */
  int indexB;

  /**
   * Index of the x-coordinate in pixel vectors.
   */
  int indexX;

  /**
   * Index of the y-coordinate in pixel vectors.
   */
  int indexY;

  /**
   * Estimated error inherent in current clustering.
   */
  double error;

  /**
   * Does the image have a current path?
   */
  bool hasPath;

  /**
   * Current path (path of last successful save).
   */
  wxString path;

  /**
   * Has the image been saved since its last change?
   */
  bool hasBeenSaved;

};

  }
}

#endif
