#include "ClusterWizard.hpp"

#include "../ClusterImage.hpp"

#include "wx/gbsizer.h"
#include "wx/event.h"

#define SPACER_SIZE 8

using namespace indii::tint;
using namespace indii::tint::gui;

ClusterWizard::ClusterWizard(wxWindow* parent) :
    wxWizard(parent, -1, _T("Tint Wizard"), wxBitmap(wxImage(_T("/usr/share/tint/images/splash.png")))) {
  image = NULL;
  clusterImage = NULL;

  /* open page */
  pageOpen = new wxWizardPageSimple(this);
  wxBoxSizer* sizerOpen = new wxBoxSizer(wxVERTICAL);
 
  wxStaticText* textImage = new wxStaticText(pageOpen, -1,
      _T("Select photo to tint."));
  fileImage = new wxFilePickerCtrl(pageOpen, ID_FILEPICKER, wxEmptyString,
      _T("Select an image"),
      _T("Supported file types ") + wxImage::GetImageExtWildcard());
  fileImage->SetWindowStyle(wxFLP_OPEN|wxFLP_FILE_MUST_EXIST);

  sizerOpen->Add(textImage, 0, wxEXPAND);
  sizerOpen->AddSpacer(SPACER_SIZE);
  sizerOpen->Add(fileImage, 0, wxEXPAND);

  pageOpen->SetSizer(sizerOpen);

  /* options page */
  pageOptions = new wxWizardPageSimple(this);
  wxGridBagSizer* sizerOptions = new wxGridBagSizer(SPACER_SIZE,SPACER_SIZE);
  pageOptions->SetSizer(sizerOptions);
  sizerOptions->AddGrowableCol(1);

  wxStaticText* labelK = new wxStaticText(pageOptions, -1,
      _T("Colour groups"));
  wxStaticText* labelIters = new wxStaticText(pageOptions, -1,
      _T("Quality"));

  spinK = new wxSpinCtrl(pageOptions, -1, _T("2"), wxDefaultPosition,
      wxDefaultSize, wxSP_ARROW_KEYS, 1, 12, 3);
  spinIters = new wxSpinCtrl(pageOptions, -1, _T("3"),
      wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 20, 10);

  sizerOptions->Add(labelK, wxGBPosition(0,0), wxDefaultSpan,
      wxALIGN_CENTRE_VERTICAL);
  sizerOptions->Add(labelIters, wxGBPosition(1,0), wxDefaultSpan,
      wxALIGN_CENTRE_VERTICAL);
  sizerOptions->Add(spinK, wxGBPosition(0,1), wxDefaultSpan,
      wxALIGN_RIGHT);
  sizerOptions->Add(spinIters, wxGBPosition(1,1), wxDefaultSpan,
      wxALIGN_RIGHT);

  checkColour = new wxCheckBox(pageOptions, -1, _T("Colour"));
  checkX = new wxCheckBox(pageOptions, -1, _T("X-coordinates"));
  checkY = new wxCheckBox(pageOptions, -1, _T("Y-coordinates"));

  checkColour->SetValue(true);

  wxStaticBoxSizer* sizerBox = new wxStaticBoxSizer(wxVERTICAL, pageOptions,
      _T("Consider..."));

  sizerBox->Add(checkColour, 0, wxEXPAND);
  sizerBox->AddSpacer(SPACER_SIZE);
  sizerBox->Add(checkX, 0, wxEXPAND);
  sizerBox->AddSpacer(SPACER_SIZE);
  sizerBox->Add(checkY, 0, wxEXPAND);

  sizerOptions->AddSpacer(SPACER_SIZE);
  sizerOptions->Add(sizerBox, wxGBPosition(2,0), wxGBSpan(1,2), wxEXPAND);

  /* progress page */
  pageProgress = new wxWizardPageSimple(this);
  wxBoxSizer* sizerProgress = new wxBoxSizer(wxVERTICAL);
  pageProgress->SetSizer(sizerProgress);

  labelStatus = new wxStaticText(pageProgress, -1, _T(""));
  gaugeStatus = new wxGauge(pageProgress, -1, 10);

  sizerProgress->Add(labelStatus, 0, wxEXPAND);
  sizerProgress->AddSpacer(SPACER_SIZE);
  sizerProgress->Add(gaugeStatus, 0, wxEXPAND);

  /* page order */
  pageOpen->SetNext(pageOptions);
  pageOptions->SetPrev(pageOpen);
  pageOptions->SetNext(pageProgress);

  /* disable buttons to begin with, enabled when input valid */
  wxWindow::FindWindowById(wxID_FORWARD, this)->Disable();

  /* dialogs */
  dialogCancel = new wxMessageDialog(this, _T("Are you sure you want to cancel? The clustering will stop at the next available opportunity."), _T("Confirm"), wxYES_NO|wxNO_DEFAULT|wxICON_QUESTION);
}

ClusterWizard::~ClusterWizard() {
  delete pageOpen;
  delete pageOptions;
  delete pageProgress;
  delete image; // becomes NULL as soon as we don't own it, so ok
}

void ClusterWizard::OnWizardPageChanging(wxWizardEvent& event) {
  if (event.GetPage() == pageOpen && event.GetDirection()) {
    /* must select file before progressing past open page */
    if (image == NULL) {
      event.Veto();
    }
  } else if (event.GetPage() == pageOptions && event.GetDirection()) {
    /* start clustering on arrival to progress page */
    wxWindow::FindWindowById(wxID_FORWARD, this)->Disable();
    labelStatus->SetLabel(_T("Clustering..."));
    gaugeStatus->SetRange(spinIters->GetValue() + 2);
    gaugeStatus->SetValue(0);

    clusterImage = new ClusterImage(image);
    image = NULL; // no longer own
    clusterImage->SetColourConsidered(checkColour->GetValue());
    clusterImage->SetXConsidered(checkX->GetValue());
    clusterImage->SetYConsidered(checkY->GetValue());

    thread = new ClusterWizardThread(this, clusterImage);
    thread->Create();
    thread->Run();
  }
}

void ClusterWizard::OnCancel(wxWizardEvent& event) {
  if (thread != NULL && thread->IsAlive()) {
    thread->Pause();
    if (dialogCancel->ShowModal() == wxID_YES) {
      Cancel();
    }
    thread->Resume();
    event.Veto();
  } else {
    delete clusterImage;
    clusterImage = NULL;
  }
}

void ClusterWizard::OnFinish(wxWizardEvent& event) {

}

void ClusterWizard::OnFileChosen(wxFileDirPickerEvent& WXUNUSED(event)) {
  /* destroy any existing image */
  delete image;

  /* test new image */
  image = new wxImage(fileImage->GetPath());

  /* enable forward button if image worked, ensure disabled otherwise */
  wxWindow* forward = wxWindow::FindWindowById(wxID_FORWARD, this);
  if (image->IsOk()) {
    forward->Enable();
  } else {
    delete image;
    image = NULL;
    forward->Disable();
  }
}

void ClusterWizard::OnClusterComplete(wxCommandEvent& WXUNUSED(event)) {
  thread->Wait();
  thread = NULL;

  labelStatus->SetLabel(_T("Complete"));
  gaugeStatus->SetValue(gaugeStatus->GetRange());
  wxWindow::FindWindowById(wxID_FORWARD, this)->Enable();
}

void ClusterWizard::OnClusterProgress(wxCommandEvent& WXUNUSED(event)) {
  gaugeStatus->SetValue(gaugeStatus->GetValue() + 1);
}

void ClusterWizard::OnClusterCancelled(wxCommandEvent& WXUNUSED(event)) {
  thread->Wait();
  thread = NULL;

  labelStatus->SetLabel(_T("Cancelled"));
}

bool ClusterWizard::RunWizard() {
  return wxWizard::RunWizard(pageOpen);
}

ClusterImage* ClusterWizard::GetClusterImage() {
  return clusterImage;
}

void ClusterWizard::Cancel() {
  cancelMutex.Lock();
}

bool ClusterWizard::IsCancelled() {
  if (cancelMutex.TryLock() == wxMUTEX_NO_ERROR) {
    cancelMutex.Unlock();
    return false;
  } else {
    return true;
  }
}

DEFINE_EVENT_TYPE(EVT_CLUSTER_COMPLETE)
DEFINE_EVENT_TYPE(EVT_CLUSTER_PROGRESS)
DEFINE_EVENT_TYPE(EVT_CLUSTER_CANCELLED)

BEGIN_EVENT_TABLE(ClusterWizard, wxWizard)
EVT_WIZARD_PAGE_CHANGING(wxID_ANY, ClusterWizard::OnWizardPageChanging)
EVT_WIZARD_CANCEL(wxID_ANY, ClusterWizard::OnCancel)
EVT_WIZARD_FINISHED(wxID_ANY, ClusterWizard::OnFinish)
EVT_FILEPICKER_CHANGED(ID_FILEPICKER, ClusterWizard::OnFileChosen)
EVT_COMMAND(wxID_ANY, EVT_CLUSTER_COMPLETE, ClusterWizard::OnClusterComplete)
EVT_COMMAND(wxID_ANY, EVT_CLUSTER_PROGRESS, ClusterWizard::OnClusterProgress)
EVT_COMMAND(wxID_ANY, EVT_CLUSTER_CANCELLED, ClusterWizard::OnClusterCancelled)
END_EVENT_TABLE()

ClusterWizardThread::ClusterWizardThread(ClusterWizard* wizard,
    ClusterImage* image) : wxThread(wxTHREAD_JOINABLE) {
  this->wizard = wizard;
  this->image = image;
}

ClusterWizardThread::~ClusterWizardThread() {
  //
}

void* ClusterWizardThread::Entry() {
  /* pre-condition */
  assert (this->image != NULL);

  int i;
  int iters = wizard->spinIters->GetValue();
  int k = wizard->spinK->GetValue();
  wxCommandEvent progressEvent(EVT_CLUSTER_PROGRESS);
  wxCommandEvent completeEvent(EVT_CLUSTER_COMPLETE);
  wxCommandEvent cancelledEvent(EVT_CLUSTER_CANCELLED);

  progressEvent.SetEventObject(wizard);
  completeEvent.SetEventObject(wizard);
  cancelledEvent.SetEventObject(wizard);

  image->Prepare();
  wxPostEvent(wizard->GetEventHandler(), progressEvent);
  for (i = 0; i < iters && !wizard->IsCancelled(); i++) {
    image->Cluster(k);
    wxPostEvent(wizard->GetEventHandler(), progressEvent);
  }
  if (!wizard->IsCancelled()) {
    image->Finalise();
    wxPostEvent(wizard->GetEventHandler(), progressEvent);
    wxPostEvent(wizard->GetEventHandler(), completeEvent);
  } else {
    wxPostEvent(wizard->GetEventHandler(), cancelledEvent);
  }

  return NULL;
}
