#ifndef INDII_TINT_GUI_CLUSTERWIZARD_HPP
#define INDII_TINT_GUI_CLUSTERWIZARD_HPP

#include "wx/wx.h"
#include "wx/wizard.h"
#include "wx/filepicker.h"
#include "wx/spinctrl.h"
#include "wx/thread.h"

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(EVT_CLUSTER_COMPLETE, -1)
DECLARE_EVENT_TYPE(EVT_CLUSTER_PROGRESS, -1)
DECLARE_EVENT_TYPE(EVT_CLUSTER_CANCELLED, -1)
END_DECLARE_EVENT_TYPES()

namespace indii {
  namespace tint {

    class ClusterImage;

    namespace gui {

      class ClusterWizardThread;

/**
 * Wizard for clustering image.
 *
 * @author Lawrence Murray <lawrence@indii.org>
 * @version $Rev: 13 $
 * @date $Date: 2007-08-11 13:17:58 +0100 (Sat, 11 Aug 2007) $
 */
class ClusterWizard : public wxWizard {

    friend class ClusterWizardThread;

public:
  /**
   * Constructor.
   *
   * @param parent Parent window.
   */
  ClusterWizard(wxWindow* parent = NULL);

  /**
   * Destructor.
   */
  ~ClusterWizard();

  /**
   * Page changing event handler.
   */
  void OnWizardPageChanging(wxWizardEvent& event);

  /**
   * Cancel button event handler.
   */
  void OnCancel(wxWizardEvent& event);

  /**
   * Finish button event handler.
   */
  void OnFinish(wxWizardEvent& event);

  /**
   * Image file chosen event handler.
   */
  void OnFileChosen(wxFileDirPickerEvent& event);

  /**
   * Clustering complete event handler.
   */
  void OnClusterComplete(wxCommandEvent& event);

  /**
   * Clustering progress event handler.
   */
  void OnClusterProgress(wxCommandEvent& event);

  /**
   * Clustering cancelled event handler.
   */
  void OnClusterCancelled(wxCommandEvent& event);

  /**
   * Run wizard.
   *
   * @return True if the wizard completed successfully, false if it was
   * cancelled.
   */
  bool RunWizard();

  /**
   * Get cluster image result.
   */
  ClusterImage* GetClusterImage();

private:
  /**
   * Cancel the clustering process. Only one thread should attempt this.
   */
  void Cancel();

  /**
   * Check if the clustering process has been cancelled. Thread safe.
   *
   * @return True if the clustering process has been cancelled, false
   * otherwise.
   */
  bool IsCancelled();

  /**
   * Regular image.
   */
  wxImage* image;

  /**
   * Cluster image.
   */
  ClusterImage* clusterImage;
  
  /*
   * Pages
   */
  wxWizardPageSimple* pageOpen;
  wxWizardPageSimple* pageOptions;
  wxWizardPageSimple* pageProgress;

  /*
   * Dialogs
   */
  wxMessageDialog* dialogCancel;

  /*
   * Widgets
   */
  wxFilePickerCtrl* fileImage;
  wxSpinCtrl* spinK;
  wxSpinCtrl* spinIters;
  wxCheckBox* checkColour;
  wxCheckBox* checkX;
  wxCheckBox* checkY;
  wxStaticText* labelStatus;
  wxGauge* gaugeStatus;

  /*
   * Threads.
   */
  ClusterWizardThread* thread;
  wxMutex cancelMutex;

  /*
   * Widget ids.
   */
  enum {
    ID_FILEPICKER = 1
  };

  DECLARE_EVENT_TABLE()

};

/**
 * Thread for clustering process.
 *
 * @author Lawrence Murray <lawrence@indii.org>
 * @version $Rev: 13 $
 * @date $Date: 2007-08-11 13:17:58 +0100 (Sat, 11 Aug 2007) $
 */
class ClusterWizardThread : public wxThread {
public:
  /**
   * Constructor.
   *
   * @param wizard Owning wizard.
   * @param image Image to cluster. Caller maintains ownership.
   */
  ClusterWizardThread(ClusterWizard* wizard, ClusterImage* image);

  /**
   * Destructor.
   */
  virtual ~ClusterWizardThread();

  /**
   * Thread work.
   *
   * Posts EVT_CLUSTER_PROGRESS event back to owning wizard after image is
   * prepared, after each clustering pass, and after image is finalised. Posts
   * EVT_CLUSTER_COMPLETE upon completion, or EVT_CLUSTER_CANCELLED if the
   * user cancels the process prematurely.
   */
  virtual void* Entry();

private:
  /**
   * Image to cluster.
   */
  ClusterImage* image;

  /**
   * Owning wizard.
   */
  ClusterWizard* wizard;

};

    }
  }
}

#endif
