#include "Workspace.hpp"

#include "ClusterWizard.hpp"
#include "ClusterImageViewer.hpp"
#include "ClusterImageEditor.hpp"
#include "../ClusterImage.hpp"

using namespace indii::tint;
using namespace indii::tint::gui;

Workspace::Workspace(ClusterImage* image) : wxFrame((wxFrame*)NULL, -1,
    _T("Tint")), image(image) {
  /* menu */
  wxMenu *menuFile = new wxMenu();
  menuFile->Append(ID_NEW, _T("&New..."));
  menuFile->AppendSeparator();
  menuFile->Append(ID_SAVE, _T("&Save"));
  menuFile->Append(ID_SAVEAS, _T("Save &As..."));
  menuFile->AppendSeparator();
  menuFile->Append(ID_CLOSE, _T("&Close"));

  wxMenu *menuView = new wxMenu();
  menuView->Append(ID_ZOOMIN, _T("Zoom &In"));
  menuView->Append(ID_ZOOMOUT, _T("Zoom &Out"));
  menuView->Append(ID_ZOOMNORMAL, _T("Zoom &Normal"));

  wxMenu *menuHelp = new wxMenu();
  menuHelp->Append(ID_ABOUT, _T("A&bout"));

  wxMenuBar *menuBar = new wxMenuBar();
  menuBar->Append(menuFile, _T("&File"));
  menuBar->Append(menuView, _T("&View"));
  menuBar->Append(menuHelp, _T("&Help"));

  /* keyboard shortcuts */
  wxAcceleratorEntry entries[4];
  entries[0].Set(wxACCEL_CTRL,  (int)'N',     ID_NEW);
  entries[1].Set(wxACCEL_CTRL,  (int)'S',     ID_SAVE);
  entries[1].Set(wxACCEL_CTRL,  (int)'+',     ID_ZOOMIN);
  entries[1].Set(wxACCEL_CTRL,  (int)'-',     ID_ZOOMOUT);
  wxAcceleratorTable accel(4, entries);

  SetAcceleratorTable(accel);
  SetMenuBar(menuBar);

  /* toolbar */
  this->CreateToolBar(wxTB_HORIZONTAL|wxTB_FLAT|wxTB_TEXT|wxTB_NODIVIDER);
  wxToolBar* toolBar = this->GetToolBar();
  toolBar->SetToolBitmapSize(wxSize(22,22));

  wxBitmap iconNew(wxImage(_T("/usr/share/tint/images/nuvola/wizard.png")));
  wxBitmap iconSave(wxImage(_T("/usr/share/tint/images/nuvola/filesave.png")));
  wxBitmap iconSaveAs(wxImage(_T("/usr/share/tint/images/nuvola/filesaveas.png")));
  wxBitmap iconZoomIn(wxImage(_T("/usr/share/tint/images/nuvola/viewmag+.png")));
  wxBitmap iconZoomOut(wxImage(_T("/usr/share/tint/images/nuvola/viewmag-.png")));
  wxBitmap iconZoomNormal(wxImage(_T("/usr/share/tint/images/nuvola/viewmag1.png")));

  toolBar->AddTool(ID_NEW, _T("New"), iconNew);
  toolBar->AddSeparator();
  toolBar->AddTool(ID_SAVE, _T("Save"), iconSave);
  toolBar->AddTool(ID_SAVEAS, _T("Save As"), iconSaveAs);
  toolBar->AddSeparator();
  toolBar->AddTool(ID_ZOOMIN, _T("Zoom In"), iconZoomIn);
  toolBar->AddTool(ID_ZOOMOUT, _T("Zoom Out"), iconZoomOut);
  toolBar->AddTool(ID_ZOOMNORMAL, _T("Zoom Normal"), iconZoomNormal);
  toolBar->Realize();

  /* dialogs */
  dialogSaveAs = new wxFileDialog(this, _T("Save As"), ::wxGetCwd(),
      wxEmptyString,
      _T("Supported file types ") + wxImage::GetImageExtWildcard(),
      wxFD_SAVE|wxOVERWRITE_PROMPT);
  dialogSaveError = new wxMessageDialog(this,
      _T("Could not save image"), _T("Error"), wxOK|wxICON_ERROR);
  dialogSavePrompt = new wxMessageDialog(this,
      _T("Save image before closing?"), _T("Save?"),
      wxYES_NO|wxYES_DEFAULT|wxICON_QUESTION);

  /* AUI manager */
  manager = new wxAuiManager(this);

  wxAuiPaneInfo viewerPaneInfo, editorPaneInfo;

  /* viewer */
  viewer = new ClusterImageViewer(this, image);
  viewerPaneInfo.DefaultPane();
  viewerPaneInfo.Centre();
  viewerPaneInfo.CloseButton(false);
  viewerPaneInfo.CaptionVisible(false);
  manager->AddPane(viewer, viewerPaneInfo);

  /* mixer */
  editor = new ClusterImageEditor(this, image, viewer);
  editorPaneInfo.DefaultPane();
  editorPaneInfo.Caption(_T("Mixer"));
  editorPaneInfo.BestSize(editor->GetBestSize());
  editorPaneInfo.MinSize(-1, editor->GetBestSize().GetHeight());
  editorPaneInfo.Bottom();
  editorPaneInfo.CloseButton(false);
  editorPaneInfo.PinButton(true);
  manager->AddPane(editor, editorPaneInfo);

  SetSize(wxSize(-1, 540));

  manager->Update();
}

Workspace::~Workspace() {  
  delete image;
}

bool Workspace::Save(bool as) {
  if (as || !image->HasPath()) {
    while (dialogSaveAs->ShowModal() == wxID_OK) {
      if (image->SaveFile(dialogSaveAs->GetPath())) {
        return true;
      } else {
        dialogSaveError->ShowModal();
      }
    }
    return false;
  } else if (!image->SaveFile()) {
    dialogSaveError->ShowModal();
    return false;
  } else {
    return true;
  }
}

void Workspace::OnNew(wxCommandEvent& WXUNUSED(event)) {
  ClusterWizard* dialogWizard = new ClusterWizard(this);

  if (dialogWizard->RunWizard()) {
    Workspace* workspace = new Workspace(dialogWizard->GetClusterImage());
    workspace->Show();
  }
  dialogWizard->Destroy();
}

void Workspace::OnSave(wxCommandEvent& WXUNUSED(event)) {
  Save();
}

void Workspace::OnSaveAs(wxCommandEvent& WXUNUSED(event)) {
  Save(true);
}

void Workspace::OnClose(wxCommandEvent& WXUNUSED(event)) {
  Close();
}

void Workspace::OnSystemClose(wxCloseEvent& event) {
  if (event.CanVeto() && !image->HasBeenSaved() &&
      dialogSavePrompt->ShowModal() == wxID_YES && !Save()) {
    event.Veto();
  } else {
    Destroy();
  }
}

void Workspace::OnZoomIn(wxCommandEvent& WXUNUSED(event)) {
  viewer->ZoomIn();
  viewer->Refresh();
}

void Workspace::OnZoomOut(wxCommandEvent& WXUNUSED(event)) {
  viewer->ZoomOut();
  viewer->Refresh();
}

void Workspace::OnZoomNormal(wxCommandEvent& WXUNUSED(event)) {
  viewer->ZoomNormal();
  viewer->Refresh();
}

void Workspace::OnAbout(wxCommandEvent& WXUNUSED(event)) {
    wxMessageBox(_T("Tint\nwww.indii.org/software/tint"),
      _T("About"), wxOK | wxICON_INFORMATION, this);
}

BEGIN_EVENT_TABLE(Workspace, wxFrame)
EVT_CLOSE(Workspace::OnSystemClose)
EVT_MENU(ID_NEW, Workspace::OnNew)
EVT_MENU(ID_SAVE, Workspace::OnSave)
EVT_MENU(ID_SAVEAS, Workspace::OnSaveAs)
EVT_MENU(ID_CLOSE, Workspace::OnClose)
EVT_MENU(ID_ZOOMIN, Workspace::OnZoomIn)
EVT_MENU(ID_ZOOMOUT, Workspace::OnZoomOut)
EVT_MENU(ID_ZOOMNORMAL, Workspace::OnZoomNormal)
EVT_MENU(ID_ABOUT, Workspace::OnAbout)
END_EVENT_TABLE()
